<?php

/*
 *  location: admin/controller/module
 */

require_once(DIR_SYSTEM . 'library/make_commerce/maksekeskus/vendor/autoload.php');

use Maksekeskus\Maksekeskus;

class ControllerExtensionModuleMakeCommerce extends Controller {

    /**
     * Module route
     */
    private $route = 'extension/module/make_commerce';

    /**
     * User errors list
     */
    private $error = array();

    /**
     * Calls parent's constructor method and loads MakeCommerce model.
     *
     * @param  array $registry
     * @return void                 
     */
    public function __construct($registry) 
    {
        parent::__construct($registry);

        $this->config->load('make_commerce');
        $this->load->model('sale/order');
        $this->load->model('extension/module/make_commerce');

        if ( ! $this->model_extension_module_make_commerce->isSettingTableCreated())
            return false;

        $this->settings    = $this->model_extension_module_make_commerce->getSettings();
        $this->is_test_env = $this->settings['api_environment'] !== 'live';

        $shop_id     = ($this->is_test_env) ? $this->settings['shop_id_test'] : $this->settings['shop_id_live'];
        $public_key  = ($this->is_test_env) ? $this->settings['public_key_test'] : $this->settings['public_key_live'];
        $private_key = ($this->is_test_env) ? $this->settings['private_key_test'] : $this->settings['private_key_live'];

        if (!(isset($this->settings['makecommerce_payment_sco_active'])) || ($this->settings['makecommerce_payment_sco_active']=="")) { // fix sco settings / db if sco update
            // $this->settings['makecommerce_payment_sco_active'] = 'off';
            $this->model_extension_module_make_commerce->updateSetting('makecommerce_payment_sco_active', 'on'); // defaults to on
            $this->settings = $this->model_extension_module_make_commerce->getSettings();  // reread settings
            if (!($this->model_extension_module_make_commerce->isSCOCartsTableCreated() ))    // if sco carts table does not exist,
                $this->model_extension_module_make_commerce->addModuleTables();               // create possibly missing tables
        }

        $this->api = new Maksekeskus($shop_id, $public_key, $private_key, $this->is_test_env);

        if (isset($this->request->get['store_id']))
            $this->store_id = $this->request->get['store_id'];
    }

    /**
     * Main method of the controller.
     * 
     * @return string|int|boolean                  
     */
    public function index() 
    {       
        $breadcrumbs_list = array();

        $this->load->model('extension/module/make_commerce');

        $this->language->load($this->route);
        $this->document->setTitle($this->language->get('heading_title'));
        $this->document->addStyle(HTTPS_SERVER . 'view/stylesheet/make_commerce/csspin.css');
        $this->document->addStyle(HTTPS_SERVER . 'view/stylesheet/make_commerce/make_commerce.css');

        $session_token          = $this->session->data['user_token'];
        $make_commerce_settings = $this->model_extension_module_make_commerce->getSettings();
        
        $translation_keys = array(
            'heading_title',
            'edit_text',
            'api_text',
            'payment_methods_text',
            'omniva_parcel_machines_text',
            'version_text',
            'omniva_courier_text',
            'button_save_text',
            'button_cancel_text',
            'merchant_portal_text',
            'makecommerce_net_text',
            'maksekeskus_ee_text',
            'api_tab_title_text',
            'api_tab_description_text',
            'current_environment_text',
            'see_more_about_text',
            'makecommerce_test_environment_text',
            'live_text',
            'test_text',
            'shop_id_test_text',
            'secret_key_test_text',
            'publishable_key_test_text',
            'merchant_portal_test_text',
            'shop_id_live_text',
            'secret_key_live_text',
            'publishable_key_live_text',
            'merchant_portal_live_text',
            'get_it_from_text',
        );

        // Load the Setting Model (All of the OpenCart Module & General Settings are saved using this Model)
        $this->load->model('setting/setting'); 

        // If the request method was POST and the current user can make changes, 
        // then update settings in the database.
        if (($this->request->server['REQUEST_METHOD'] == 'POST') && $this->validate())
            $this->saveSettings();

        $data['error_warning'] = (isset($this->error['warning'])) ? $this->error['warning'] : '';

        // Assign the language data for parsing it to view        
        $this->addTranslationStrings($data, $translation_keys);   

        $data['action']                          = $this->url->link('extension/module/make_commerce', 'user_token=' . $session_token, true);     
        $data['cancel']                          = $this->url->link('marketplace/extension', 'user_token=' . $session_token.'&type=module', 'SSL');
        $data['api_tab_link']                    = $this->url->link('extension/module/make_commerce', 'user_token=' . $session_token, true); 
        $data['payment_tab_link']                = $this->url->link('extension/payment/make_commerce', 'user_token=' . $session_token, true);   
        $data['omniva_parcel_machines_tab_link'] = $this->url->link('extension/shipping/omniva_parcel_by_make_commerce', 'user_token=' . $session_token, true);  
        $data['omniva_courier_tab_link']         = $this->url->link('extension/shipping/omniva_courier_by_make_commerce', 'user_token=' . $session_token, true);
        $data['ajax_check_keys_url']             = $this->url->link('extension/module/make_commerce/checkKeys'); 

        $breadcrumbs_list[] = array('text' => $this->language->get('text_home'), 'uri' => 'common/dashboard');  
        $breadcrumbs_list[] = array('text' => $this->language->get('text_extensions'), 'uri' => 'marketplace/extension', 'param' => '&type=module');
        $breadcrumbs_list[] = array('text' => $this->language->get('heading_title'), 'uri' => 'extension/module/make_commerce');

        $data['breadcrumbs']   = $this->getFormattedBreadcrumbs($breadcrumbs_list);
        $data['session_token'] = $session_token;
        $data['settings']      = $make_commerce_settings;

        $data['merchant_portal_test_url'] = 'https://merchant-test.maksekeskus.ee/api.html';
        $data['merchant_portal_live_url'] = 'https://merchant.maksekeskus.ee/api.html';
        $data['mk_version']               = $this->config->get('mk_version');

        $data['header']      = $this->load->controller('common/header');
        $data['column_left'] = $this->load->controller('common/column_left');
        $data['footer']      = $this->load->controller('common/footer');

        $this->response->setOutput($this->load->view('extension/module/make_commerce', $data));
    }

    /**
     * Checks whether the saved keys are valid.
     * 
     * @return void               
     */
    public function checkKeys()
    {
        $settings    = $this->model_extension_module_make_commerce->getSettings();
        $is_test_env = $this->request->get['apiEnvironment'] == 'test';
        $shop_id     = $this->request->get['shopId'];
        $public_key  = $this->request->get['publicKey'];
        $private_key = $this->request->get['privateKey'];
        $shop_config = array();

        $response = array(
            'status'  => 200,
            'message' => 'API connection established successfully'
        );

        $maksekeskus_api = new Maksekeskus($shop_id, $public_key, $private_key, $is_test_env);

        $request_params = array(
            'environment' => json_encode(array(
                'platform' => 'opencart ' . VERSION,
                'module'   => 'omnivaopen ' . $this->config->get('mk_version')
            ))
        );

        try {
             $maksekeskus_api->getShopConfig($request_params);
        } catch (Exception $e) {            
            $response = array(
                'status'  => $e->getCode(),
                'message' => $e->getMessage()
            );
        }

        print json_encode($response);
        exit;
    }

    /**
     * Gets order details for MakeCommerce refund form.
     *
     * @return void                 
     */
    public function getOrderDetails()
    {
        if ( ! array_key_exists('orderId', $this->request->get)) {
            $response = array(
                'status'  => 404,
                'message' => 'Order ID is required',
                'content' => array()
            );

            $this->response->setOutput(json_encode($response));
        }

        $response = array(
            'status'  => 200,
            'message' => 'Payment methods has been successfully updated',
            'content' => array()
        );

        $order_id       = $this->request->get['orderId'];
        $order_info     = $this->model_sale_order->getOrder($order_id);
        $transaction_id = $this->model_extension_module_make_commerce->getOrderData($order_id, 'transaction_id');

        if (empty($transaction_id)) {
            $response = array(
                'status'  => 404,
                'message' => 'Transaction not found',
                'content' => array()
            );

            $this->response->setOutput(json_encode($response));
        }

        $transaction = $this->api->getTransaction($transaction_id);

        if (isset($transaction->refunded_amount)) {
            $refunded = $transaction->refunded_amount;
            $refunded = $refunded;
        } else {
            $refunded = 0.00;
        }

        $total = (float) $transaction->amount - (float) $refunded;

        $response['content'] = array(
            'total'    => $total . ' €',
            'amount'   => number_format((float)$total, 2, '.', ''),
            'refunded' => '-' . number_format((float)$refunded, 2, '.', '') . ' €' 
        );

        $this->response->setOutput(json_encode($response));
    }

    /**
     * TODO: Add some better validation.
     * 
     * Makes a refund by calling MakeCommerce's API.
     *
     * @return void                 
     */
    public function refund() 
    {
        $order_id = $this->request->get['orderId'];
        $amount   = $this->request->get['amount'];
        $comment  = $this->request->get['comment'];
        $user_id  = $this->session->data['user_id'];

        $formatted_amount = $amount . ' €';
        $transaction_id   = $this->model_extension_module_make_commerce->getOrderData($order_id, 'transaction_id');

        $response = array(
            'status'  => 200,
            'message' => $formatted_amount . ' successfully refunded'
        );

        if (empty($transaction_id)) {
            $response = array(
                'status'  => 403,
                'message' => 'This order cannot be refunded using MakeCommerce service'
            );

            $this->response->setOutput(json_encode($response));
        }

        try {
            $api_refund_response = $this->api->createRefund(
                $transaction_id, 
                array('amount' => $amount, 'comment' => ($comment ? : 'refund')));          

            if (isset($api_refund_response->status) && ($api_refund_response->status === 'CREATED' || $api_refund_response->status === 'SETTLED')) {                
                $transaction     = $api_refund_response->transaction;
                $order_info      = $this->model_sale_order->getOrder($order_id);
                $order_status_id = $order_info['order_status_id'];
                $refunded        = $transaction->refunded_amount;
                $refund          = $this->model_extension_module_make_commerce->addRefund($order_id, $amount, $comment, $user_id, $transaction_id, $api_refund_response->id);

                $total = (float) $transaction->amount - (float) $transaction->refunded_amount;

                if ($refund !== false)
                    $this->addOrderHistory($refund, $order_id, $order_status_id, $comment, $formatted_amount);

                $response['total']    = $total . ' €';
                $response['amount']   = number_format((float)$total, 2, '.', '');
                $response['refunded'] = '-' . number_format((float)$refunded, 2, '.', '') . ' €';

            }
        } catch(Exception $e) {
            $error_code = $e->getCode();
            $response   = $this->getErrorResponse($error_code);

            error_log($e->getMessage());
        }

        $this->response->setOutput(json_encode($response));
    }

    /**
     * Registers shipment at carrier systems.
     *
     * @return void                 
     */
    public function registerShipment()
    {
        if (empty($this->request->get['orderId'])) {
            print('Order ID not found!');
            exit();
        }

        $order_id = $this->request->get['orderId'];    
        $order    = $this->model_sale_order->getOrder($order_id);

        if (empty($order)) {
            print('Order not found!');
            exit();
        }

        $adequate_shipping_codes = array('omniva_parcel', 'omniva_courier_by_make_commerce');        

        if (strpos($order['shipping_code'], ':') !== false)
            $shipping_code = explode(':', $order['shipping_code']);   
        else
            $shipping_code = explode('.', $order['shipping_code']);

        if ($shipping_code[0] == 'omniva_parcel_by_make_commerce')
            $shipping_code[0] = 'omniva_parcel';

        if ( ! isset($shipping_code[0]) || ! in_array($shipping_code[0], $adequate_shipping_codes)) {
            print('This is not an adequate order!');
            exit();
        }

        $shipping_request = array('credentials' => array(), 'orders' => array());
        $shipping_type    = (strpos($shipping_code[0], 'omniva_courier') !== false) ? 'omniva_courier': $shipping_code[0];
        $shipping_company = (strpos($shipping_type, 'omniva') !== false) ? 'omniva': $shipping_type;
        $carrier_uc       = mb_strtoupper($shipping_company);
        $m_service_type;

        if ($shipping_type === 'omniva_parcel')
            $m_service_type = 'PA';
        else if ($shipping_type === 'omniva_courier')
            $m_service_type = $this->settings['omniva_courier_register_shipments_as'];

        $shipping_request['credentials'][$carrier_uc] = array(
            'carrier'  => $carrier_uc, 
            'username' => $this->getShippingSetting('service_user', $shipping_type), 
            'password' => $this->getShippingSetting('service_password', $shipping_type)
        );

        $sender = array(
            'name'  => $this->getShippingSetting('shop_name', $shipping_type),
            'phone' => $this->getShippingSetting('shop_phone', $shipping_type),
            'email' => $this->getShippingSetting('shop_email', $shipping_type)
        );

        if ($shipping_company == 'omniva') {
            $sender['country']    = $this->getShippingSetting('shop_address_country', $shipping_type);
            $sender['city']       = $this->getShippingSetting('shop_address_city', $shipping_type);
            $sender['street']     = $this->getShippingSetting('shop_address_street', $shipping_type);
            $sender['postalCode'] = $this->getShippingSetting('shop_postal_code', $shipping_type);
        }

        $sr_order = array(
            'carrier'   => $carrier_uc,
            'orderId'   => $order['order_id'],
            'recipient' => array(
                'name'  => ($order['shipping_firstname'] ? $order['shipping_firstname'] . ' ' . $order['shipping_lastname'] : $order['firstname'] . ' ' . $order['lastname'] ),
                'phone' => $order['telephone'], 
                'email' => $order['email']),
            'sender'      => $sender,
            'destination' => array(),
        );

        if ($shipping_company == 'omniva') {
            $sr_order['destination'] = array(
                'postalCode' => $order['shipping_postcode'],
                'country'    => $order['shipping_iso_code_2'],
                'county'     => '',
                'city'       => $order['shipping_city'],
                'street'     => $order['shipping_address_1'] . ' ' . $order['shipping_address_2']
            );
        }

        if ( ! empty($order['shipping_code']) && ($shipping_type == 'omniva_parcel')) {
            if (strpos($order['shipping_code'], ':') !== false)
                $shipping_code = explode(':', $order['shipping_code']);   
            else
                $shipping_code = explode('.', $order['shipping_code']);

            if (isset($shipping_code[1]))
                $sr_order['destination']['destinationId'] = $shipping_code[1];
        }

        if ( ! empty($m_service_type))
            $sr_order['services'] = array('serviceType' => $m_service_type);

        $shipping_request['orders'][] = $sr_order;
        $shipping_request['credentials'] = array_values($shipping_request['credentials']);

        try {
            $response = $this->api->createShipments($shipping_request);                                
        } catch (Exception $e) {
            error_log($e->getMessage());
            
            print($e->getMessage());
            exit();
        }

        $shipments = ( ! empty($response->shipments)) ? $response->shipments : $response;
        $manifest =  ( ! empty($response->manifests)) ? $response->manifests[0] : false;

        foreach ($shipments as $shipment) {
            if ($manifest)
                $this->model_extension_module_make_commerce->updateOrderData(
                    $order['order_id'], 
                    'parcel_machine_manifest', 
                    $this->sanitizeTextField($manifest)
                );
            if ( ! empty($shipment->orderId) && ! empty($shipment->shipmentId)) {
                $this->model_extension_module_make_commerce->updateOrderData(
                    $order['order_id'], 
                    'parcel_machine_shipment_id', 
                    $this->sanitizeTextField($shipment->shipmentId)
                );
                $this->model_extension_module_make_commerce->deleteOrderData($order['order_id'], 'parcel_machine_error');
            } else if ( ! empty($shipment->orderId) && ! empty($shipment->barCode)) {
                $this->model_extension_module_make_commerce->updateOrderData(
                    $order['order_id'], 
                    'parcel_machine_shipment_id', 
                    $this->sanitizeTextField($shipment->barCode)
                );

                $this->model_extension_module_make_commerce->deleteOrderData($order['order_id'], '_parcel_machine_error');
            } else if ( ! empty($shipment->orderId) && ! empty($shipment->errorMessage)) {
                $this->model_module_make_commerce->updateOrderData(
                    $order['order_id'], 
                    'parcel_machine_error', 
                    $this->sanitizeTextField($shipment->errorMessage)
                );
            }
        }

        print('200');
        exit();
    }

    /**
     * Generates parcel labels for shipments registered at carriers.
     *
     * @return void                 
     */
    public function printParcelLabel()
    {
        if (empty($this->request->get['order_id']) && empty($this->request->post['selected'])) {
            die('One or multiple IDs are required required!');
        }

        if ( ! empty($this->request->get['order_id'])) {
            $order_ids = array($this->request->get['order_id']);   
        } else {
            $order_ids = $this->request->post['selected'];
        }

        $adequate_shipping_codes = array('omniva_parcel', 'omniva_courier_by_make_commerce');   
        $shipping_request        = array('credentials' => array(), 'orders' => array(), 'printFormat' => 'A4');

        foreach ($order_ids as $order_id) {
            $order       = $this->model_sale_order->getOrder($order_id);
            $shipment_id = $this->model_extension_module_make_commerce->getOrderData($order_id, 'parcel_machine_shipment_id');

            if (empty($order) || empty($shipment_id))
                continue;

            if (strpos($order['shipping_code'], ':') !== false)
                $shipping_code = explode(':', $order['shipping_code']);   
            else
                $shipping_code = explode('.', $order['shipping_code']);

            if ($shipping_code[0] == 'omniva_parcel_by_make_commerce')
                $shipping_code[0] = 'omniva_parcel';

            if ( ! isset($shipping_code[0]) || ! in_array($shipping_code[0], $adequate_shipping_codes))
                continue;     

            $shipping_type    = (strpos($shipping_code[0], 'omniva_courier') !== false) ? 'omniva_courier': $shipping_code[0];
            $shipping_company = (strpos($shipping_type, 'omniva') !== false) ? 'omniva': $shipping_type;
            $carrier_uc       = mb_strtoupper($shipping_company);
            $m_service_type   = null; 

            if ($shipping_type === 'omniva_parcel')
                $m_service_type = 'PA';
            else if ($shipping_type === 'omniva_courier')
                $m_service_type = $this->settings['omniva_courier_register_shipments_as'];

            $shipping_request['credentials'][$carrier_uc] = array(
                'carrier'  => $carrier_uc, 
                'username' => $this->getShippingSetting('service_user', $shipping_type), 
                'password' => $this->getShippingSetting('service_password', $shipping_type)
            );

            $sender = array(
                'name'  => $this->getShippingSetting('shop_name', $shipping_type),
                'phone' => $this->getShippingSetting('shop_phone', $shipping_type),
                'email' => $this->getShippingSetting('shop_email', $shipping_type)
            );

            if ($shipping_company == 'omniva') {
                $sender['country']    = $this->getShippingSetting('shop_address_country', $shipping_type);
                $sender['city']       = $this->getShippingSetting('shop_address_city', $shipping_type);
                $sender['street']     = $this->getShippingSetting('shop_address_street', $shipping_type);
                $sender['postalCode'] = $this->getShippingSetting('shop_postal_code', $shipping_type);
            }

            $sr_order = array(
                'carrier'   => $carrier_uc,
                'orderId'   => $order['order_id'],
                'recipient' => array(
                    'name'  => ($order['shipping_firstname'] ? $order['shipping_firstname'] . ' ' . $order['shipping_lastname'] : $order['firstname'] . ' ' . $order['lastname'] ),
                    'phone' => $order['telephone'], 
                    'email' => $order['email']),
                'sender'      => $sender,
                'shipmentId'  => $shipment_id,
                'destination' => array(),
            );

            if ($shipping_company == 'omniva') {
                $sr_order['destination'] = array(
                    'postalCode' => $order['shipping_postcode'],
                    'country'    => $order['shipping_iso_code_2'],
                    'county'     => '',
                    'city'       => $order['shipping_city'],
                    'street'     => $order['shipping_address_1'] . ' ' . $order['shipping_address_2']
                );
            }

            if ( ! empty($order['shipping_code']) && ($shipping_type == 'omniva_parcel')) {
                if (strpos($order['shipping_code'], ':') !== false)
                    $shipping_code = explode(':', $order['shipping_code']);   
                else
                    $shipping_code = explode('.', $order['shipping_code']);

                if (isset($shipping_code[1]))
                    $sr_order['destination']['destinationId'] = $shipping_code[1];
            }

            if ( ! empty($m_service_type))
                $sr_order['services'] = array('serviceType' => $m_service_type);

            $shipping_request['orders'][] = $sr_order;
        }

        $shipping_request['credentials'] = array_values($shipping_request['credentials']);

        try {
            $response = $this->api->createLabels($shipping_request);

            error_log(print_r($response, 1));
        } catch (Exception $e) {
            echo $e->getMessage();
            exit();
        }

        if (empty($response->labelUrl)) {
            return;
        }

        $this->response->redirect($response->labelUrl);
    }

    /**
     * This method is called when the module gets installed.
     * 
     * @return void                 
     */
    public function install() 
    {
        $this->model_extension_module_make_commerce->addModuleTables();
        $this->model_extension_module_make_commerce->addDefaultSettings();
    }

    /**
     * Appends refund status to the order's history.
     *
     * @param  array  $refund
     * @param  int    $order_id
     * @param  int    $order_status_id
     * @param  string $comment
     * @param  string $refund_amount
     * @return void                 
     */
    private function addOrderHistory($refund, $order_id, $order_status_id, $comment, $refund_amount)
    {
        $this->load->model('user/user');

        $user        = $this->model_user_user->getUser($refund['user_id']);
        $username    = (isset($user['username'])) ? $user['username'] : '';
        $refund_time = date('d-m-Y H:i:s', strtotime($refund['time']));

        $refund_comment = 'Refund <strong>#' . $refund['mk_refund_id'] .'</strong> - ' . $refund_time . ' by <strong>' . $username . '</strong>'
                        . '<br /> <span style="color: #f56b6b;"> -' . $refund_amount . '</span>'
                        . '<br /> ' . $comment;

        $data = array(
            'append'          => 1,
            'notify'          => false,
            'order_status_id' => $order_status_id,
            'comment'         => $refund_comment
        );

        $this->model_extension_module_make_commerce->addOrderHistory($order_id, $data); 
    }

    /**
     * Gets settings for specific shipping method. 
     *
     * @param  string $name
     * @param  string $type
     * @return string                 
     */
    private function getShippingSetting($name, $type)
    {
        if (isset($this->settings[$type . '_' . $name]))
            return $this->settings[$type . '_' . $name];
        else
            return '';
    }

    /**
     * Sanitizes a string from user input or from the database.
     *
     * @param  string $string
     * @return string                 
     */
    private function sanitizeTextField($string) 
    {
        if (strpos($string, '<') !== false) {
            $string = $this->preKsesLessThan($string);
            // This will strip extra whitespace for us.
            $string = $this->stripAllTags($string, true);
        } else {
            $string = trim(preg_replace('/[\r\n\t ]+/', ' ', $string));
        }

        $found = false;

        while (preg_match('/%[a-f0-9]{2}/i', $string, $match)) {
            $string = str_replace($match[0], '', $string);
            $found = true;
        }

        if ($found) {
            // Strip out the whitespace that may now exist after removing the octets.
            $string = trim(preg_replace('/ +/', ' ', $string));
        }

        return $string;
    }

    /**
     * Convert lone less than signs.
     *
     * @param  string $text
     * @return string
     */
    private function preKsesLessThan($text) 
    {
        return preg_replace_callback('%<[^>]*?((?=<)|>|$)%', array($this, 'preKsesLessThanCallback'), $text);
    }

    /**
     * Callback function used by preg_replace.
     *
     * @param  array  $matches 
     * @return string
     */
    private function preKsesLessThanCallback($matches) 
    {
        if (false === strpos($matches[0], '>'))
            return esc_html($matches[0]);

        return $matches[0];
    }

    /**
     * Properly strip all HTML tags including script and style.
     *
     * @param  string $string        
     * @param  bool   $remove_breaks 
     * @return string 
     */
    private function stripAllTags($string, $remove_breaks = false) 
    {
        $string = preg_replace('@<(script|style)[^>]*?>.*?</\\1>@si', '', $string);
        $string = strip_tags($string);

        if ($remove_breaks)
            $string = preg_replace('/[\r\n\t ]+/', ' ', $string);

        return trim($string);
    }

    /**
     * Gets an error message and status code for the given error code.
     *
     * @param  string $error_code
     * @return array                 
     */
    private function getErrorResponse($error_code)
    {
        $response = array();

        switch ($error_code) {
            case '1001':
                $response = array(
                    'status'  => 403,
                    'message' => 'Refund amount is not valid.'
                );
                break;
            case '1040':
                $response = array(
                    'status'  => 403,
                    'message' => 'Could not create refund. This usually happens while transaction is in process. 
                                    Please wait a few minutes and try again'
                );
                break;
            case '1041':
                $response = array(
                    'status'  => 403,
                    'message' => 'Transaction in wrong status, cannot be refunded!'
                );
                break;
            case '1042':
                $response = array(
                    'status'  => 403,
                    'message' => 'Refund amount exceeds available amount!'
                );
                break;
            default:
                $response = array(
                    'status'  => 500,
                    'message' => 'This order cannot be refunded using MakeCommerce service. 
                                    Something unexpected happened. Please contact MakeCommerce service.'
                );
                break;
        }

        return $response;
    }

    /**
     * Checks if the user has premission to make changes.
     * 
     * @return boolean                 
     */
    private function validate() 
    {
        $module_route = 'extension/module/make_commerce';

        // Check the user permission to manipulate the module
        if ( ! $this->user->hasPermission('modify', $module_route))
            $this->error['warning'] = $this->language->get('error_permission');

        if ($this->error)
            return false;

        return true;  
    }

    /**
     * Updates MakeCommerce settings in the database.
     * 
     * @return void                 
     */
    private function saveSettings()
    {
        $this->model_extension_module_make_commerce->editSettings($this->request->post, 'api');

        $this->response->redirect($this->url->link($this->route, 'user_token=' . $this->session->data['user_token'], 'SSL'));
    }

    /**
     * Appends translation strings to the data array.
     *
     * @param  array $data
     * @param  array $translation_keys
     * @return void                 
     */
    private function addTranslationStrings(&$data, $translation_keys)
    {
        foreach ($translation_keys as $translation_key)
            $data[$translation_key] = $this->language->get($translation_key);
    }
    
    /**
     * Gets a list of formatted breadcrumbs for templates.
     * 
     * @param  array $breadcrumbs_list
     * @return array                 
     */
    private function getFormattedBreadcrumbs($breadcrumbs_list)
    {
        $breadcrumbs = array();

        foreach ($breadcrumbs_list as $breadcrumbs_item) {
            $params = (isset($breadcrumbs_item['param'])) ? $breadcrumbs_item['param'] : '';

            $breadcrumbs[] = array(
                'text' => $breadcrumbs_item['text'],
                'href' => $this->url->link($breadcrumbs_item['uri'], 'user_token=' . $this->session->data['user_token'] . $params, true)
            );
        }

        return $breadcrumbs;
    }

}
