<?php

/*
 *  location: admin/controller/payment
 */

require_once(DIR_SYSTEM . 'library/make_commerce/maksekeskus/vendor/autoload.php');

use Maksekeskus\Maksekeskus;

class ControllerExtensionPaymentMakeCommerce extends Controller {

    /**
     * Module route
     */
    private $route = 'extension/payment/make_commerce';

    /**
     * User errors list
     */
    private $error = array();

    /**
     * Calls parent's constructor method and loads MakeCommerce model.
     *
     * @param  array $registry
     * @return void                 
     */
    public function __construct($registry) 
    {
        parent::__construct($registry);
        $this->load->model('extension/module/make_commerce');
        $this->config->load('make_commerce');

        if (isset($this->request->get['store_id']))
            $this->store_id = $this->request->get['store_id'];
    }

    /**
     * Main controller method.
     * 
     * @return void                 
     */
    public function index() 
    {
        $this->load->model('extension/module/make_commerce'); 

        $this->language->load($this->route);
        $this->document->setTitle($this->language->get('heading_title'));
        $this->document->addStyle(HTTPS_SERVER . 'view/stylesheet/make_commerce/csspin.css');
        $this->document->addStyle(HTTPS_SERVER . 'view/stylesheet/make_commerce/make_commerce.css');

        $this->load->model('localisation/language');

        $session_token          = $this->session->data['user_token'];
        $make_commerce_settings = $this->model_extension_module_make_commerce->getSettings(); 
        $languages              = $this->model_localisation_language->getLanguages();

        $translation_keys = array(
            'heading_title',
            'edit_text',
            'api_text',
            'payment_methods_text',
            'omniva_parcel_machines_text',
            'version_text',
            'omniva_courier_text',
            'button_save_text',
            'button_cancel_text',
            'merchant_portal_text',
            'makecommerce_net_text',
            'maksekeskus_ee_text',
            'current_environment_text',
            'payment_tab_title_text',
            'general_text',
            'enable_text',
            'disable_text',
            'enable_makecommerce_payments_text',
            'enable_makecommerce_payments_sco_text',
            'user_interface_text',
            'set_as_default_selection_text',
            'makecommerce_widget_default_text',
            'payments_widget_title_text',
            'payments_widget_title_tooltip_text',
            'payment_sco_tos_text',
            'payment_sco_tos_tooltip_text',
            'payment_channel_logo_title_text',
            'small_text',
            'medium_text',
            'large_text',
            'hide_country_selector_text',
            'define_custom_order_text',
            'define_custom_order_tooltip_text',
            'credit_card_settings_text',
            'prefill_credit_card_form_text',
            'prefill_credit_card_form_tooltip_text',
            'advanced_settings_text',
            'update_payment_methods_text',
            'update_payment_methods_tooltip_text',
            'update_text',
            'update_url_text',
            'update_url_tooltip_text',
            'last_time_updated_text',
        );

        // Load the Setting Model (All of the OpenCart Module & General Settings are saved using this Model)
        $this->load->model('setting/setting'); 

        if (in_array('ajax_update_payment_methods', $this->request->get))
            $this->ajaxUpdatePaymentMethods($make_commerce_settings);

        // If the request method was POST and the current user can make changes, 
        // then update settings in the database.
        if (($this->request->server['REQUEST_METHOD'] == 'POST') && $this->validate())
            $this->saveSettings();

        $data['error_warning'] = (isset($this->error['warning'])) ? $this->error['warning'] : '';

        // Assign the language data for parsing it to view        
        $this->addTranslationStrings($data, $translation_keys);          

        $data['action']                          = $this->url->link('extension/payment/make_commerce', 'user_token=' . $session_token, true);     
        $data['cancel']                          = $this->url->link('marketplace/extension', 'user_token=' . $session_token.'&type=payment', 'SSL');
        $data['api_tab_link']                    = $this->url->link('extension/module/make_commerce', 'user_token=' . $session_token, true); 
        $data['payment_tab_link']                = $this->url->link('extension/payment/make_commerce', 'user_token=' . $session_token, true);  
        $data['omniva_parcel_machines_tab_link'] = $this->url->link('extension/shipping/omniva_parcel_by_make_commerce', 'user_token=' . $session_token, true);  
        $data['omniva_courier_tab_link']         = $this->url->link('extension/shipping/omniva_courier_by_make_commerce', 'user_token=' . $session_token, true);     
        $data['ajax_update_url']                 = $this->url->link('extension/payment/make_commerce');

        $breadcrumbs_list[] = array('text' => $this->language->get('text_home'), 'uri' => 'common/dashboard'); 
        $breadcrumbs_list[] = array('text' => $this->language->get('text_payments'), 'uri' => 'marketplace/extension', 'param' => '&type=payment');
        $breadcrumbs_list[] = array('text' => $this->language->get('heading_title'), 'uri' => 'extension/payment/make_commerce');

        $data['breadcrumbs']   = $this->getFormattedBreadcrumbs($breadcrumbs_list);
        $data['settings']      = $make_commerce_settings;
        $data['session_token'] = $session_token;
        $data['languages']     = $languages;
        $data['mk_version']    = $this->config->get('mk_version');

        if ( ! empty($make_commerce_settings['payment_updated_timestamp']))
            $data['payment_methods_timestamp'] = date('d/m/y | H:i:s', $make_commerce_settings['payment_updated_timestamp']);
        else 
            $data['payment_methods_timestamp'] = '-';

        $data['header']      = $this->load->controller('common/header');
        $data['column_left'] = $this->load->controller('common/column_left');
        $data['footer']      = $this->load->controller('common/footer');

        $this->response->setOutput($this->load->view('extension/payment/make_commerce', $data));
    }

    /**
     * Fetches shop configuration from the API and stores the fetched data
     * into the database.
     *
     * @param  array $settings
     * @return void                 
     */
    public function ajaxUpdatePaymentMethods($settings)
    {
        $is_test_env = $settings['api_environment'] !== 'live';
        $shop_id     = ($is_test_env) ? $settings['shop_id_test'] : $settings['shop_id_live'];
        $public_key  = ($is_test_env) ? $settings['public_key_test'] : $settings['public_key_live'];
        $private_key = ($is_test_env) ? $settings['private_key_test'] : $settings['private_key_live'];
        $shop_config = array();

        $response = array(
            'status'  => 200,
            'message' => 'Payment methods has been successfully updated'
        );

        $maksekeskus_api = new Maksekeskus($shop_id, $public_key, $private_key, $is_test_env);

        $request_params = array(
            'environment' => json_encode(array(
                'platform' => 'opencart ' . VERSION,
                'module'   => 'omnivaopen ' . $this->config->get('mk_version')
            ))
        );

        try {
            $shop_config = $maksekeskus_api->getShopConfig($request_params);
            $payment_methods = $shop_config->payment_methods;
            $parcel_machines = $maksekeskus_api->getDestinations(array('type' => 'APT'));
        } catch (Exception $e) {
            $response = array(
                'status'  => $e->getCode(),
                'message' => $e->getMessage()
            );
        }

        if (isset($payment_methods->banklinks))
            $this->model_extension_module_make_commerce->editPaymentMethodFields($payment_methods->banklinks, 'banklink');

        if (isset($payment_methods->cards))
            $this->model_extension_module_make_commerce->editPaymentMethodFields($payment_methods->cards, 'card');

        if (isset($parcel_machines))
            $this->model_extension_module_make_commerce->editShippingParcels($parcel_machines);

        $shop_name = (isset($shop_config->name) && strlen($shop_config->name)) ? $shop_config->name : '';
        $current_timestamp = time();

        $this->model_extension_module_make_commerce->editSettings(array(
                'shop_name' => $shop_name,
                'payment_updated_timestamp' => $current_timestamp
            ), 'shop'
        );

        $response['last_time_updated'] = date('d/m/y | H:i:s', $current_timestamp);

        print json_encode($response);
        exit;
    }

    /**
     * This method is called when the extension gets installed.
     * 
     * @return void                 
     */
    public function install() 
    {
        $this->model_extension_module_make_commerce->addModuleTables();
        $this->model_extension_module_make_commerce->addDefaultSettings();
    }

    /**
     * Checks if the user has premission to make changes.
     * 
     * @return boolean                 
     */
    private function validate() 
    {
        $module_route = 'extension/payment/make_commerce';

        // Check the user permission to manipulate the module
        if ( ! $this->user->hasPermission('modify', $module_route))
            $this->error['warning'] = $this->language->get('error_permission');

        if ($this->error)
            return false;

        return true;  
    }

    /**
     * Appends translation strings to the data array.
     *
     * @param  array $data
     * @param  array $translation_keys
     * @return void                 
     */
    private function addTranslationStrings(&$data, $translation_keys)
    {
        foreach ($translation_keys as $translation_key)
            $data[$translation_key] = $this->language->get($translation_key);
    }

    /**
     * Updates MakeCommerce settings in the database.
     * 
     * @return void                 
     */
    private function saveSettings()
    {   
        $this->request->post['payments_widget_language_title'] = $this->getMethodNamesJsonFromRequest();

        if (array_key_exists('makecommerce_payment_active', $this->request->post) && $this->request->post['makecommerce_payment_active'] == 'on') {
            $setting['payment_make_commerce_status'] = 1;
            if (array_key_exists('makecommerce_payment_sco_active', $this->request->post) && $this->request->post['makecommerce_payment_sco_active'] == 'on') {
                $setting['payment_make_commerce_sco_status'] = 1;
            } else {
                $setting['payment_make_commerce_sco_status'] = 1;
            }
            $this->model_setting_setting->editSetting('payment_make_commerce', $setting);
        } else {
            $this->model_setting_setting->editSetting('payment_make_commerce', array('payment_make_commerce_status' => 0, 'payment_make_commerce_sco_status' => 0));
        }


        $this->model_extension_module_make_commerce->editSettings($this->request->post, 'payment');
        $this->response->redirect($this->url->link($this->route, 'user_token=' . $this->session->data['user_token'], 'SSL'));
    }

    private function getMethodNamesJsonFromRequest()
    {
        $result = array();
        $post   = $this->request->post;

        $method_names       = (isset($post['payments_widget_titles']) ? $post['payments_widget_titles'] : array());
        $method_names_codes = (isset($post['payments_widget_titles_codes']) ? $post['payments_widget_titles_codes'] : array());

        foreach ($method_names_codes as $method_names_code)
            $result[$method_names_code] = (isset($method_names[$method_names_code]) ? $method_names[$method_names_code] : '');

        return json_encode($result);
    }
    
    /**
     * Gets a list of formatted breadcrumbs for templates.
     * 
     * @param  array $breadcrumbs_list
     * @return array                 
     */
    private function getFormattedBreadcrumbs($breadcrumbs_list)
    {
        $breadcrumbs = array();

        foreach ($breadcrumbs_list as $breadcrumbs_item) {
            $params = (isset($breadcrumbs_item['param'])) ? $breadcrumbs_item['param'] : '';

            $breadcrumbs[] = array(
                'text' => $breadcrumbs_item['text'],
                'href' => $this->url->link($breadcrumbs_item['uri'], 'user_token=' . $this->session->data['user_token'] . $params, true)
            );
        }

        return $breadcrumbs;
    }

}
