<?php

/*
 *  location: admin/controller/extension/shipping
 */

class ControllerExtensionShippingOmnivaParcelByMakeCommerce extends Controller {

    /**
     * Shipping extension route
     */
    private $route = 'extension/shipping/omniva_parcel_by_make_commerce';

    /**
     * Calls parent's constructor method and loads MakeCommerce model.
     *
     * @param  array $registry
     * @return void                 
     */
    public function __construct($registry) 
    {
        parent::__construct($registry);
        $this->load->model('extension/module/make_commerce');
        $this->config->load('make_commerce');

        if (isset($this->request->get['store_id']))
            $this->store_id = $this->request->get['store_id'];
    }

    /**
     * Main controller method.
     * 
     * @return void                 
     */
    public function index() 
    {
        $this->load->language($this->route);
        $this->document->setTitle($this->language->get('heading_title'));
        $this->document->addStyle(HTTPS_SERVER . 'view/stylesheet/make_commerce/make_commerce.css');
        $this->document->addScript(HTTPS_SERVER . 'view/javascript/make_commerce/jquery.numeric.min.js');

        $this->load->model('localisation/language');

        $session_token          = $this->session->data['user_token'];
        $make_commerce_settings = $this->model_extension_module_make_commerce->getSettings();
        $languages              = $this->model_localisation_language->getLanguages();

        $translation_keys = array(
            'heading_title',
            'edit_text',
            'api_text',
            'payment_methods_text',
            'omniva_parcel_machines_text',
            'version_text',
            'omniva_courier_text',
            'button_save_text',
            'button_cancel_text',
            'merchant_portal_text',
            'makecommerce_net_text',
            'maksekeskus_ee_text',
            'general_text',
            'enable_text',
            'disable_text',
            'enable_makecommerce_omniva_parcel_text',
            'maximum_weight_allowed_text',
            'look_and_feel_options_text',
            'shipping_method_title_text',
            'prioritize_text',
            'parcel_prioritization_description_text',
            'parcel_short_office_names_description_text',
            'short_names_text',
            'options_for_presentation_text',
            'api_access_for_omniva_text',
            'omniva_web_services_username_text',
            'omniva_web_services_password_text',
            'return_address_text',
            'shop_name_text',
            'shop_phone_text',
            'shop_email_text',
            'shop_address_country_text',
            'shop_address_description_text',
            'shop_address_city_text',
            'shop_address_street_text',
            'shop_postal_code_text',
            'omniva_api_access_description_text',
            'tax_class_text',
            'none_text',
            'sort_order_text',
            'pricing_text',
            'type_text',
            'fixed_text',
            'return_address_desc_text',
            'per_geo_zones_text',
            'shipping_cost_text',
            'geo_zones_text',
            'enable_omniva_parcel_in_zone_text',
            'shipping_price_text',
            'shipping_price_tip_text',
            'free_shipping_amount_text',
            'free_shipping_amount_tip_text',
            'no_geo_zones_found_text',
        );

        // Load the Setting Model (All of the OpenCart Module & General Settings are saved using this Model)
        $this->load->model('setting/setting'); 

        $this->load->model('localisation/geo_zone');
        $this->load->model('localisation/tax_class');

        // If the request method was POST and the current user can make changes, 
        // then update settings in the database.
        if (($this->request->server['REQUEST_METHOD'] == 'POST') && $this->validate())
            $this->saveSettings();

        $data['error_warning'] = (isset($this->error['warning'])) ? $this->error['warning'] : '';

        // Assign the language data for parsing it to view        
        $this->addTranslationStrings($data, $translation_keys);     

        $data['action']                          = $this->url->link('extension/shipping/omniva_parcel_by_make_commerce', 'user_token=' . $session_token, true);     
        $data['cancel']                          = $this->url->link('marketplace/extension', 'user_token=' . $session_token.'&type=shipping', 'SSL');
        $data['api_tab_link']                    = $this->url->link('extension/module/make_commerce', 'user_token=' . $session_token, true); 
        $data['payment_tab_link']                = $this->url->link('extension/payment/make_commerce', 'user_token=' . $session_token, true);  
        $data['omniva_parcel_machines_tab_link'] = $this->url->link('extension/shipping/omniva_parcel_by_make_commerce', 'user_token=' . $session_token, true);
        $data['omniva_courier_tab_link']         = $this->url->link('extension/shipping/omniva_courier_by_make_commerce', 'user_token=' . $session_token, true);      

        $breadcrumbs_list[] = array('text' => $this->language->get('text_home'), 'uri' => 'common/dashboard');  
        $breadcrumbs_list[] = array('text' => $this->language->get('text_shipping'), 'uri' => 'marketplace/extension', 'param' => '&type=shipping');
        $breadcrumbs_list[] = array('text' => $this->language->get('heading_title'), 'uri' => 'extension/shipping/omniva_parcel_by_make_commerce');

        $data['breadcrumbs']   = $this->getFormattedBreadcrumbs($breadcrumbs_list);
        $data['settings']      = $make_commerce_settings;
        $data['session_token'] = $session_token;

        $data['geo_zones']   = $this->model_localisation_geo_zone->getGeoZones();
        $data['languages']   = $languages;
        $data['tax_classes'] = $this->model_localisation_tax_class->getTaxClasses();
        $data['mk_version']  = $this->config->get('mk_version');
       
        $data['header']      = $this->load->controller('common/header');
        $data['column_left'] = $this->load->controller('common/column_left');
        $data['footer']      = $this->load->controller('common/footer');

        $this->response->setOutput($this->load->view('extension/shipping/omniva_parcel_by_make_commerce', $data));
    }

    /**
     * This method is called when the extension gets installed.
     * 
     * @return void                 
     */
    public function install() 
    {
        $this->model_extension_module_make_commerce->addModuleTables();
        $this->model_extension_module_make_commerce->addDefaultSettings();
    }

    /**
     * Checks if the user has premission to make changes.
     * 
     * @return boolean                 
     */
    private function validate() 
    {
        $module_route = 'extension/shipping/omniva_parcel_by_make_commerce';

        // Check the user permission to manipulate the module
        if ( ! $this->user->hasPermission('modify', $module_route))
            $this->error['warning'] = $this->language->get('error_permission');

        if ($this->error)
            return false;

        return true;  
    }

    /**
     * Updates MakeCommerce settings in the database.
     * 
     * @return void                 
     */
    private function saveSettings()
    {
        $sort = '';
        $this->request->post['omniva_geo_zone_data'] = $this->getGeoZonesJsonFromRequest();
        $this->request->post['omniva_parcel_method_name_languages'] = $this->getMethodNamesJsonFromRequest();

        if (array_key_exists('makecommerce_omniva_parcel_active', $this->request->post) && 
            $this->request->post['makecommerce_omniva_parcel_active'] == 'on')
            $status = 1;
        else
            $status = 0;

        if (array_key_exists('omniva_parcel_sort_order', $this->request->post))
            $sort = (int)$this->request->post['omniva_parcel_sort_order'];    

        $this->model_setting_setting->editSetting(
            'shipping_omniva_parcel_by_make_commerce', 
            array(
                'shipping_omniva_parcel_by_make_commerce_status' => $status,
                'shipping_omniva_parcel_by_make_commerce_sort_order' => $sort
            )
        );
        
        $this->model_extension_module_make_commerce->editSettings($this->request->post, 'omniva_parcel');
 
        $this->response->redirect($this->url->link($this->route, 'user_token=' . $this->session->data['user_token'], 'SSL'));
    }

    /**
     * Appends translation strings to the data array.
     *
     * @param  array $data
     * @param  array $translation_keys
     * @return void                 
     */
    private function addTranslationStrings(&$data, $translation_keys)
    {
        foreach ($translation_keys as $translation_key)
            $data[$translation_key] = $this->language->get($translation_key);
    }

    /**
     * Generates a json string from the geo zone request variables.
     *
     * @return string                 
     */
    private function getGeoZonesJsonFromRequest()
    {
        $result = array();
        $post   = $this->request->post;

        $zone_ids             = (isset($post['omniva_parcel_zone_ids']) ? $post['omniva_parcel_zone_ids'] : array());
        $shipping_costs       = (isset($post['omniva_parcel_costs']) ? $post['omniva_parcel_costs'] : array());
        $free_shipping_amount = (isset($post['omniva_parcel_free_shipping_amounts']) ? $post['omniva_parcel_free_shipping_amounts'] : array());        
        $geo_zones_active     = (isset($post['omniva_parcel_geo_zones_active']) ? $post['omniva_parcel_geo_zones_active'] : array());

        foreach ($zone_ids as $zone_id) {
            $result[$zone_id]['omniva_parcel_geo_zone_active']      = (isset($geo_zones_active[$zone_id]) ? $geo_zones_active[$zone_id] : '');
            $result[$zone_id]['omniva_parcel_cost']                 = (isset($shipping_costs[$zone_id]) ? $shipping_costs[$zone_id] : '');
            $result[$zone_id]['omniva_parcel_free_shipping_amount'] = (isset($free_shipping_amount[$zone_id]) ? $free_shipping_amount[$zone_id] : '');
            
        }

        return json_encode($result);
    }

    /**
     * Generates a json string from method names request variables.
     *
     * @return string                 
     */
    private function getMethodNamesJsonFromRequest()
    {
        $result = array();
        $post   = $this->request->post;

        $method_names       = (isset($post['omniva_parcel_method_names']) ? $post['omniva_parcel_method_names'] : array());
        $method_names_codes = (isset($post['omniva_parcel_method_names_codes']) ? $post['omniva_parcel_method_names_codes'] : array());

        foreach ($method_names_codes as $method_names_code) {
            $result[$method_names_code] = (isset($method_names[$method_names_code]) ? $method_names[$method_names_code] : '');
        }

        return json_encode($result);
    }

    /**
     * Gets a list of formatted breadcrumbs for templates.
     * 
     * @param  array $breadcrumbs_list
     * @return array                 
     */
    private function getFormattedBreadcrumbs($breadcrumbs_list)
    {
        $breadcrumbs = array();

        foreach ($breadcrumbs_list as $breadcrumbs_item) {
            $params = (isset($breadcrumbs_item['param'])) ? $breadcrumbs_item['param'] : '';

            $breadcrumbs[] = array(
                'text' => $breadcrumbs_item['text'],
                'href' => $this->url->link($breadcrumbs_item['uri'], 'user_token=' . $this->session->data['user_token'] . $params, true)
            );
        }

        return $breadcrumbs;
    }

}
