<?php

/*
 *  location: admin/model/extension/module
 */
require_once(DIR_SYSTEM . 'library/make_commerce/cron/cron_tasks.php');
class ModelExtensionModuleMakeCommerce extends Model {

    /**
     * Module API settings
     */
    private $api_settings;

    /**
     * Module payment settings
     */
    private $payment_settings;

    /**
     * Omniva parcel machine settings
     */
    private $omniva_parcel_settings;

    /**
     * Omniva courier settings
     */
    private $omniva_courier_settings;

    /**
     * MakeCommerce shop settings (https://makecommerce.net)
     */
    private $shop_settings;

    /**
     * Module default settings
     */
    private $default_settings;

    /**
     * Calls parent's constructor method.
     *
     * @return void
     */
    public function __construct($registry)
    {
        parent::__construct($registry);

        $this->api_settings = array(
            'api_environment',
            'shop_id_test',
            'private_key_test',
            'public_key_test',
            'shop_id_live',
            'private_key_live',
            'public_key_live',
        );

        $this->payment_settings = array(
            'makecommerce_payment_active',
            'makecommerce_payment_sco_active',
            'payment_sco_tos_link',
            'default_selection',
            'payments_widget_language_title',
            'payment_logo_size',
            'hide_country_selector',
            'custom_order_payment_channels',
            'prefill_credit_card_form'
        );

        $this->shop_settings = array('shop_name', 'payment_updated_timestamp');

        $this->omniva_parcel_settings = array(
            'makecommerce_omniva_parcel_active',
            'omniva_parcel_tax_class',
            'omniva_parcel_pricing_type',
            'omniva_parcel_shipping_cost',
            'omniva_parcel_fixed_free_shipping_amount',
            'omniva_geo_zone_data',
            'omniva_parcel_sort_order',
            'omniva_parcel_maximum_weight',
            'omniva_parcel_method_name',
            'omniva_parcel_method_name_languages',
            'omniva_parcel_prioritization',
            'omniva_parcel_short_office_names',
            'omniva_parcel_service_user',
            'omniva_parcel_service_password',
            'omniva_parcel_shop_name',
            'omniva_parcel_shop_phone',
            'omniva_parcel_shop_email',
            'omniva_parcel_shop_address_country',
            'omniva_parcel_shop_address_city',
            'omniva_parcel_shop_address_street',
            'omniva_parcel_shop_postal_code'
        );

        $this->omniva_courier_settings = array(
            'makecommerce_omniva_courier_active',
            'omniva_courier_tax_class',
            'omniva_courier_pricing_type',
            'omniva_courier_shipping_cost',
            'omniva_courier_fixed_free_shipping_amount',
            'omniva_courier_geo_zone_data',
            'omniva_courier_sort_order',
            'omniva_courier_maximum_weight',
            'omniva_courier_method_name_languages',
            'omniva_courier_method_sub_name',
            'omniva_courier_method_sub_name_languages',
            'omniva_courier_register_shipments_as',
            'omniva_courier_service_user',
            'omniva_courier_service_password',
            'omniva_courier_shop_name',
            'omniva_courier_shop_phone',
            'omniva_courier_shop_email',
            'omniva_courier_shop_address_country',
            'omniva_courier_shop_address_city',
            'omniva_courier_shop_address_street',
            'omniva_courier_shop_postal_code'
        );

        $this->default_settings = array(
            'api_environment'                           => 'test',
            'payment_logo_size'                         => 'large',
            'prefill_credit_card_form'                  => 'on',
            'omniva_parcel_maximum_weight'              => 0,
            'omniva_parcel_pricing_type'                => 'fixed',
            'omniva_parcel_shipping_cost'               => 2.99,
            'omniva_parcel_fixed_free_shipping_amount'  => 0,
            'omniva_parcel_prioritization'              => 'on',
            'omniva_courier_maximum_weight'             => 0,
            'omniva_courier_pricing_type'               => 'fixed',
            'omniva_courier_shipping_cost'              => 2.99,
            'omniva_courier_fixed_free_shipping_amount' => 0,
            'omniva_courier_register_shipments_as'      => 'PK'
        );
    }

    /**
     * Checks if the make_commerce_settings table is creted.
     *
     * @return boolean
     */
    public function isSettingTableCreated()
    {
        $is_table_created = true;

        $query_obj = $this->db->query("SHOW TABLES LIKE '%make_commerce_settings'");

        if ($query_obj->num_rows < 1)
            $is_table_created = false;

        return $is_table_created;
    }

    /**
     * Checks if the make_commerce_sco_carts table is creted.
     *
     * @return boolean
     */
    public function isSCOCartsTableCreated()
    {
        $is_table_created = true;

        $query_obj = $this->db->query("SHOW TABLES LIKE '%make_commerce_sco_carts'");

        if ($query_obj->num_rows < 1)
            $is_table_created = false;

        return $is_table_created;
    }

    /**
     * Updates specific setting.
     *
     * @param  string $name
     * @param  string $value
     * @return void
     */
    public function updateSetting($name, $value)
    {

		$this->db->query("UPDATE `" . DB_PREFIX . "make_commerce_settings`
				SET `value` = '" . $this->db->escape($value) . "'
				WHERE name = '" . $this->db->escape($name) . "'");
    }

    /**
     * Edit multiple settings at once.
     *
     * @param  array $values
     * @param  array $setting_type
     * @return void
     */
    public function editSettings($values, $setting_type = false)
    {
        $setting_names = $this->getSettingNames($setting_type);
        $counter       = 0;

        $query = "INSERT INTO `" . DB_PREFIX . "make_commerce_settings` (name, value) VALUES ";

        foreach ($values as $key => $value) {
            if ( ! in_array($key, $setting_names))
                continue;

            if ($counter > 0)
                $query .= ", ";

            $query .= "('" . $key . "','" . $this->db->escape($value) . "')";

            $counter++;
        }

        $query .= ";";

        $this->deleteSettings($setting_names);
        $this->db->query($query);
    }

    /**
     * Adds a meta value to the given order.
     *
     * @param  int    $order_id
     * @param  string $name
     * @param  string $value
     * @return void
     */
    public function updateOrderData($order_id, $name, $value)
    {
        $select_query_obj = $this->db->query("SELECT id FROM `" . DB_PREFIX . "make_commerce_orders_data` WHERE order_id='$order_id' AND name='$name';");

        if ($select_query_obj->num_rows > 0) {
            $entry_id = $select_query_obj->row['id'];

            $query = "UPDATE `" . DB_PREFIX . "make_commerce_orders_data` SET `value` = '" . $this->db->escape($value) . "' WHERE id = '" . $entry_id . "'";
        } else {
            $query = "INSERT INTO `" . DB_PREFIX . "make_commerce_orders_data` (order_id, name, value) VALUES ('$order_id', '$name', '{$this->db->escape($value)}')";
        }

        $this->db->query($query);
    }

    /**
     * Deletes a meta value from the given order.
     *
     * @param  int    $order_id
     * @param  string $name
     * @return void
     */
    public function deleteOrderData($order_id, $name)
    {
        $this->db->query("DELETE FROM `" . DB_PREFIX . "make_commerce_orders_data` WHERE order_id='$order_id' AND name='$name';");
    }

    /**
     * Gets specific order's meta value.
     *
     * @param  int    $order_id
     * @param  string $name
     * @return string | array
     */
    public function getOrderData($order_id, $name = false)
    {
        $query = "SELECT name, value FROM `" . DB_PREFIX . "make_commerce_orders_data` WHERE order_id='$order_id'";

        if ($name !== false) {
            $result = '';
            $query .= " AND name='$name';";
        } else {
            $result = array();
            $query .= ";";
        }

        $query_obj = $this->db->query($query);

        if ($query_obj->num_rows > 0 && $name !== false)
            foreach ($query_obj->rows as $row)
                $result = $row['value'];
        elseif ($query_obj->num_rows > 0 && $name === false)
              foreach ($query_obj->rows as $row)
                $result[$row['name']] = $row['value'];

        return $result;
    }

    /**
     * Appends a new refund and returns the newly inserted DB row.
     *
     * @param  int    $order_id
     * @param  float  $amount
     * @param  string $comment
     * @param  int    $user_id
     * @param  string $transaction_id
     * @param  int    $mk_refund_id
     * @return array | false
     */
    public function addRefund($order_id, $amount, $comment, $user_id, $transaction_id, $mk_refund_id)
    {
        $this->db->query("INSERT INTO `" . DB_PREFIX . "make_commerce_refunds`
                (order_id, amount, comment, user_id, transaction_id, mk_refund_id)
                VALUES ('$order_id', '$amount', '$comment', '$user_id', '$transaction_id', '$mk_refund_id')");

        $select_query_obj = $this->db->query("SELECT * FROM `" . DB_PREFIX . "make_commerce_refunds` WHERE id=LAST_INSERT_ID()");

        if ($select_query_obj->num_rows > 0)
            return $select_query_obj->row;

        return false;
    }

    /**
     * Edit multiple payment method settings at once.
     *
     * @param  array  $fields
     * @param  string $type
     * @return void | false
     */
    public function editPaymentMethodFields($fields, $type)
    {
        if (empty($fields))
            return false;

        $counter = 0;

        $query = "INSERT INTO `" . DB_PREFIX . "make_commerce_payment_methods` (type, country_iso_code, name, url) VALUES ";

        foreach ($fields as $field) {
            $country = (isset($field->country)) ? $field->country : '';
            $name    = (isset($field->name)) ? $field->name : '';
            $url     = (isset($field->url)) ? $field->url : '';

            if ($counter > 0)
                $query .= ", ";

            $query .= "('" . $type . "','" .
                $this->db->escape($country) . "','" .
                $this->db->escape($name) . "','" .
                $this->db->escape($url) .
            "')";

            $counter++;
        }

        $query .= ";";

        $this->deletePaymentMethodFields($type);
        $this->db->query($query);
    }

    /**
     * Edit multiple shipping parcels at once.
     *
     * @param  array $parcels
     * @return void
     */
    public function editShippingParcels($parcels)
    {
        $counter = 0;

        $query = "INSERT INTO `" . DB_PREFIX . "make_commerce_shipping_parcels`
                (carrier,
                 type,
                 name,
                 city,
                 country_iso_code,
                 availability,
                 temp_service_hours,
                 temp_service_hours_until,
                 temp_service_hours_2,
                 temp_service_hours_2_until,
                 parcel_id,
                 address,
                 zip,
                 x,
                 y)
                VALUES ";

        foreach ($parcels as $parcel) {
            $carrier                   = (isset($parcel->carrier)) ? strtolower($parcel->carrier) : '';
            $type                      = (isset($parcel->type)) ? strtolower($parcel->type) : '';
            $name                      = (isset($parcel->name)) ? $parcel->name : '';
            $city                      = (isset($parcel->city)) ? $parcel->city : '';
            $country_iso_code          = (isset($parcel->country)) ? strtolower($parcel->country) : '';
            $availability              = (isset($parcel->availability)) ? $parcel->availability : '';
            $temp_service_hours        = (isset($parcel->tempServiceHours)) ? $parcel->tempServiceHours : '';
            $temp_service_hours_until  = (isset($parcel->tempServiceHoursUntil)) ? $parcel->tempServiceHoursUntil : '';
            $temp_service_hours2       = (isset($parcel->tempServiceHours2)) ? $parcel->tempServiceHours2 : '';
            $temp_service_hours2_until = (isset($parcel->tempServiceHours2Until)) ? $parcel->tempServiceHours2Until : '';
            $parcel_id                 = (isset($parcel->id)) ? $parcel->id : '';
            $address                   = (isset($parcel->address)) ? $parcel->address : '';
            $zip                       = (isset($parcel->zip)) ? $parcel->zip : '';
            $x                         = (isset($parcel->x)) ? $parcel->x : '';
            $y                         = (isset($parcel->y)) ? $parcel->y : '';

            if ($counter > 0)
                $query .= ", ";

            $query .= "(
                '" . $this->db->escape($carrier) . "','" .
                $this->db->escape($type) . "','" .
                $this->db->escape($name) . "','" .
                $this->db->escape($city) . "','" .
                $this->db->escape($country_iso_code) . "','" .
                $this->db->escape($availability) . "','" .
                $this->db->escape($temp_service_hours) . "','" .
                $this->db->escape($temp_service_hours_until) . "','" .
                $this->db->escape($temp_service_hours2) . "','" .
                $this->db->escape($temp_service_hours2_until) . "','" .
                $this->db->escape($parcel_id) . "','" .
                $this->db->escape($address) . "','" .
                $this->db->escape($zip) . "','" .
                $this->db->escape($x) . "','" .
                $this->db->escape($y) . "'" .
            ")";

            $counter++;
        }

        $query .= ";";

        $this->deleteShippingParcels();
        $this->db->query($query);
    }

    /**
     * Removes the given settings from the database.
     *
     * @param  array $setting_names
     * @return void
     */
    public function deleteSettings($setting_names)
    {
        $counter = 0;

        $query = "DELETE FROM `" . DB_PREFIX . "make_commerce_settings`";

        foreach ($setting_names as $setting_name) {
            if ($counter == 0)
                $query .= " WHERE name = '" . $setting_name . "'";
            else
                 $query .= " OR name = '" . $setting_name . "'";

            $counter++;
        }

        $this->db->query($query);
    }

    /**
     * Removes all specific type settings from the database.
     *
     * @param  string $type
     * @return void
     */
    public function deletePaymentMethodFields($type)
    {
        $query = "DELETE FROM `" . DB_PREFIX . "make_commerce_payment_methods` WHERE type = '" . $type . "'";

        $this->db->query($query);
    }

    /**
     * Removes all shipping parcels from the database.
     *
     * @return void
     */
    public function deleteShippingParcels()
    {
        $query = "DELETE FROM `" . DB_PREFIX . "make_commerce_shipping_parcels`";

        $this->db->query($query);
    }

    /**
     * Gets an array of all MakeCommerce settings that are stored in the database.
     *
     * @return array
     */
    public function getSettings()
    {
        $result = array();
        $query_obj = $this->db->query("SELECT name, value FROM `" . DB_PREFIX . "make_commerce_settings`");

        foreach ($query_obj->rows as $row)
            $result[$row['name']] = $row['value'];

        return $this->formatSettings($result);
    }

    /**
     * Gets an array of MakeCommerce payment methods.
     *
     * @return array
     */
    public function getPaymentMethods()
    {
        $result              = array();
        $result['banklinks'] = array();
        $result['cards']     = array();
        $result['countries'] = array();

        $query_obj = $this->db->query("SELECT * FROM `" . DB_PREFIX . "make_commerce_payment_methods`");

        foreach ($query_obj->rows as $row) {
            if ( ! empty($row['country_iso_code']) && ! in_array($row['country_iso_code'], $result['countries']))
                $result['countries'][] = $row['country_iso_code'];

            if ($row['type'] == 'banklink')
                $result['banklinks'][] = $row;
            elseif ($row['type'] == 'card')
                $result['cards'][] = $row;
        }

        return $result;
    }

    /**
     * Gets an array of MakeCommerce shipping parcels data.
     *
     * @param  string $carrier
     * @return array
     */
    public function getShippingParcels($carrier = '')
    {
        $result = array();

        $query = "SELECT * FROM `" . DB_PREFIX . "make_commerce_shipping_parcels` WHERE type='apt'";

        if ( ! empty($carrier))
            $query .= " AND carrier='$carrier'";

        $query_obj = $this->db->query($query);

        if ($query_obj->num_rows < 1)
            return $result;

        $result['ee'] = array();
        $result['lv'] = array();
        $result['lt'] = array();

        foreach ($query_obj->rows as $row) {
            if (strtolower($row['country_iso_code']) == 'ee')
                $result['ee'][$row['city']][] = $row;
            elseif (strtolower($row['country_iso_code']) == 'lv')
                $result['lv'][$row['city']][] = $row;
            elseif (strtolower($row['country_iso_code']) == 'lt')
                $result['lt'][$row['city']][] = $row;
        }

        return $result;
    }

    /**
     * Adds module settings tables.
     *
     * @return void
     */
    public function addModuleTables()
    {
        $this->db->query("CREATE TABLE IF NOT EXISTS `" . DB_PREFIX . "make_commerce_settings` (
          `id` int(11) NOT NULL AUTO_INCREMENT,
          `store_id` int(11) NOT NULL,
          `name` varchar(100) NOT NULL,
          `value` text NOT NULL,
          PRIMARY KEY (`id`)
        ) ENGINE=MyISAM DEFAULT CHARSET=utf8 COLLATE=utf8_general_ci;");

        $this->db->query("CREATE TABLE IF NOT EXISTS `" . DB_PREFIX . "make_commerce_payment_methods` (
          `id` int(11) NOT NULL AUTO_INCREMENT,
          `type` varchar(50) NOT NULL,
          `country_iso_code` CHAR(2),
          `name` varchar(50) NOT NULL,
          `url` varchar(250) NOT NULL,
          PRIMARY KEY (`id`)
        ) ENGINE=MyISAM DEFAULT CHARSET=utf8 COLLATE=utf8_general_ci;");

        $this->db->query("CREATE TABLE IF NOT EXISTS `" . DB_PREFIX . "make_commerce_shipping_parcels` (
          `id` int(11) NOT NULL AUTO_INCREMENT,
          `carrier` varchar(100) NOT NULL,
          `type` varchar(50) NOT NULL,
          `name` varchar(200) NOT NULL,
          `city` varchar(100) NOT NULL,
          `country_iso_code` CHAR(2) NOT NULL,
          `availability` varchar(150),
          `temp_service_hours` varchar(150),
          `temp_service_hours_until` varchar(100),
          `temp_service_hours_2` varchar(150),
          `temp_service_hours_2_until` varchar(100),
          `parcel_id` int(20) NOT NULL,
          `address` varchar(150) NOT NULL,
          `zip` int(10) NOT NULL,
          `x` varchar(50),
          `y` varchar(50),
          PRIMARY KEY (`id`)
        ) ENGINE=MyISAM DEFAULT CHARSET=utf8 COLLATE=utf8_general_ci;");

        $this->db->query("CREATE TABLE IF NOT EXISTS `" . DB_PREFIX . "make_commerce_orders_data` (
          `id` int(11) NOT NULL AUTO_INCREMENT,
          `order_id` int(11) NOT NULL,
          `name` varchar(100) NOT NULL,
          `value` text NOT NULL,
          `added_on` TIMESTAMP DEFAULT CURRENT_TIMESTAMP,
          PRIMARY KEY (`id`)
        ) ENGINE=MyISAM DEFAULT CHARSET=utf8 COLLATE=utf8_general_ci;");

        $this->db->query("CREATE TABLE IF NOT EXISTS `" . DB_PREFIX . "make_commerce_refunds` (
          `id` int(11) NOT NULL AUTO_INCREMENT,
          `order_id` int(11) NOT NULL,
          `amount` varchar(100) NOT NULL,
          `comment` text NOT NULL,
          `user_id` int(11) NOT NULL,
          `transaction_id` varchar(100) NOT NULL,
          `mk_refund_id` varchar(100) NOT NULL,
          `time` DATETIME,
          PRIMARY KEY (`id`)
        ) ENGINE=MyISAM DEFAULT CHARSET=utf8 COLLATE=utf8_general_ci;");

        $this->db->query("CREATE TABLE IF NOT EXISTS `" . DB_PREFIX . "make_commerce_sco_carts` (
          `id` int(11) unsigned not null auto_increment,
          `cart_id` varchar(64) not null default '',
          `order_id` int(10) unsigned not null,
          `created` int(10) unsigned not null,
          `modified` int(10) unsigned not null,
          `status` tinyint unsigned not null,
          `content` mediumtext,
           unique key id (`id`),
           key cart_id (`cart_id`)
        ) ENGINE=MyISAM DEFAULT CHARSET=utf8 COLLATE=utf8_general_ci;");

        // Check if makecommerce_no_parcel_machine column exists. If it doesn't, then add it to the table.
        $query_obj = $this->db->query("SHOW COLUMNS FROM `" . DB_PREFIX . "product` LIKE 'makecommerce_no_parcel_machine'");

        if ($query_obj->num_rows < 1)
            $this->db->query("ALTER TABLE `" . DB_PREFIX . "product` ADD `makecommerce_no_parcel_machine` varchar(50)");

         // Check if makecommerce_no_shipping_cost column exists. If it doesn't, then add it to the table.
        $query_obj = $this->db->query("SHOW COLUMNS FROM `" . DB_PREFIX . "product` LIKE 'makecommerce_no_shipping_cost'");

        if ($query_obj->num_rows < 1)
            $this->db->query("ALTER TABLE `" . DB_PREFIX . "product` ADD `makecommerce_no_shipping_cost` varchar(50)");

        // IMPORTANT!!!
        // We change session column datatype because default datatype size limit is too low.
        $this->db->query("ALTER TABLE `" . DB_PREFIX . "session` MODIFY `data` LONGTEXT");
    }

    /**
     * Add default MakeCommerce settings.
     *
     * @return void
     */
    public function addDefaultSettings()
    {
        $default_settings = array_merge($this->default_settings, $this->getDefaultTitleSettings());

        foreach ($default_settings as $name => $value)
            $this->insertSettingIfNotExists($name, $value);
    }

    /**
     * Inserts a record into MakeCommerce settings table if it doesn't exist already.
     *
     * @param  string $name
     * @param  string $value
     * @return void
     */
    public function insertSettingIfNotExists($name, $value)
    {
        $this->db->query("INSERT INTO `" . DB_PREFIX . "make_commerce_settings` (name, value)
            SELECT '$name', '$value' FROM DUAL
        WHERE NOT EXISTS
            (SELECT id FROM `" . DB_PREFIX . "make_commerce_settings` WHERE name='$name')");
    }

    /**
     * Drops module settings tables.
     *
     * @return void
     */
    public function dropModuleTables()
    {
    	$this->db->query("DROP TABLE IF EXISTS `" . DB_PREFIX . "make_commerce_settings`");
        $this->db->query("DROP TABLE IF EXISTS `" . DB_PREFIX . "make_commerce_payment_methods`");
        $this->db->query("DROP TABLE IF EXISTS `" . DB_PREFIX . "make_commerce_shipping_parcels`");
        $this->db->query("DROP TABLE IF EXISTS `" . DB_PREFIX . "make_commerce_orders_data`");
        $this->db->query("DROP TABLE IF EXISTS `" . DB_PREFIX . "make_commerce_refunds`");
        $this->db->query("DROP TABLE IF EXISTS `" . DB_PREFIX . "oc_make_commerce_sco_carts`");
    }

    /**
     * Makes a CURL request to the API in order to add order's history.
     *
     * @param  int    $order_id
     * @param  array  $data
     * @param  int    $store_id
     * @return string
     */
    public function addOrderHistory($order_id, $data, $store_id = 0) {
        $json = array();

        $this->load->model('setting/store');

        $store_info = $this->model_setting_store->getStore($store_id);

        if ($store_info) {
            $url = $store_info['ssl'];
        } else {
            $url = HTTPS_CATALOG;
        }

        $curl = curl_init();

        // Set SSL if required
        if (substr($url, 0, 5) == 'https') {
            curl_setopt($curl, CURLOPT_PORT, 443);
        }

        curl_setopt($curl, CURLOPT_HEADER, false);
        curl_setopt($curl, CURLINFO_HEADER_OUT, true);
        curl_setopt($curl, CURLOPT_USERAGENT, $this->request->server['HTTP_USER_AGENT']);
        curl_setopt($curl, CURLOPT_SSL_VERIFYHOST, false);
        curl_setopt($curl, CURLOPT_SSL_VERIFYPEER, false);
        curl_setopt($curl, CURLOPT_FORBID_REUSE, false);
        curl_setopt($curl, CURLOPT_RETURNTRANSFER, true);
        curl_setopt($curl, CURLOPT_URL, $url . 'index.php?route=api/make_commerce/history&order_id=' . $order_id);
        curl_setopt($curl, CURLOPT_POST, true);
        curl_setopt($curl, CURLOPT_POSTFIELDS, http_build_query($data));

        $json = curl_exec($curl);

        curl_close($curl);

        return $json;
    }

    /**
     * Sets order's payment method to the given one.
     *
     * @param  int $order_id
     * @param  int $title
     * @return void
     */
    public function setOrderPaymentMethod($order_id, $title)
    {
        $this->db->query("UPDATE `" . DB_PREFIX . "order`
            SET `payment_method` = '" . $this->db->escape($title) . "'
            WHERE order_id = '" . $this->db->escape($order_id) . "'");
    }

    /**
     * Gets an array of available setting names.
     *
     * @param  string $type
     * @return array
     */
    private function getSettingNames($type = false)
    {
        if ($type === 'api')
            return $this->api_settings;
        elseif ($type === 'payment')
            return $this->payment_settings;
        elseif ($type === 'shop')
            return $this->shop_settings;
        elseif ($type === 'omniva_parcel')
            return $this->omniva_parcel_settings;
        elseif ($type === 'omniva_courier')
            return $this->omniva_courier_settings;
        elseif ($type === false)
            return array_merge(
                $this->api_settings,
                $this->payment_settings,
                $this->shop_settings,
                $this->omniva_parcel_settings,
                $this->omniva_courier_settings
            );
        else
            return array();
    }

    /**
     * Parses module settings data.
     *
     * @param  array  $saved_settings
     * @param  string $type
     * @return array
     */
    private function formatSettings($saved_settings, $type = false)
    {
        $setting_names = $this->getSettingNames($type);
        $result = array();

        foreach ($setting_names as $setting_name) {
            $settings_value = (isset($saved_settings[$setting_name])) ? $saved_settings[$setting_name] : '';

            if ($this->isJson($settings_value))
                $settings_value = json_decode($settings_value, true);

            $result[$setting_name] = $settings_value;
        }

        return $result;
    }

    /**
     * Gets an array of shipping methods and payment method default titles.
     *
     * @return array
     */
    private function getDefaultTitleSettings()
    {
        $this->load->model('localisation/language');
        $languages = $this->model_localisation_language->getLanguages();

        $payments_widget_language_title = array();
        $omniva_parcel_method_name_languages = array();
        $omniva_courier_method_name_languages = array();

        foreach ($languages as $language) {
            $payments_widget_language_title[$language['code']]                          = 'Pay with bank-links or credit card';
            $omniva_parcel_method_name_languages[$language['code']]                     = 'Omniva Parcel Machine';
            $omniva_courier_method_name_languages[$language['code']]['method_name']     = 'Omniva Courier';
            $omniva_courier_method_name_languages[$language['code']]['method_sub_name'] = 'Omniva Courier';
        }

        return array(
            'payments_widget_language_title'       => json_encode($payments_widget_language_title),
            'omniva_parcel_method_name_languages'  => json_encode($omniva_parcel_method_name_languages),
            'omniva_courier_method_name_languages' => json_encode($omniva_courier_method_name_languages),
        );

    }

    /**
     * Check whether a string is in json format.
     *
     * @param  string  $string
     * @param  boolean $return_data
     * @return boolean
     */
    private function isJson($string, $return_data = false) {
        $data = json_decode($string);

        return (json_last_error() == JSON_ERROR_NONE) ? ($return_data ? $data : TRUE) : FALSE;
    }

}
