<?php

require_once(DIR_SYSTEM . 'library/make_commerce/maksekeskus/vendor/autoload.php');

use Maksekeskus\Maksekeskus;

class ControllerExtensionModuleMakeCommerce extends Controller {

    /**
     * Loads settings and sets up properties.
     *
     * @param  array $registry
     * @return void
     */
    public function __construct($registry)
    {
        parent::__construct($registry);

        $this->config->load('make_commerce');
        $this->load->model('extension/payment/make_commerce');

        $this->payment_methods = $this->model_extension_payment_make_commerce->getPaymentMethods();
        $this->settings        = $this->model_extension_payment_make_commerce->getSettings();
        $this->is_test_env     = $this->settings['api_environment'] !== 'live';

        $shop_id     = ($this->is_test_env) ? $this->settings['shop_id_test'] : $this->settings['shop_id_live'];
        $public_key  = ($this->is_test_env) ? $this->settings['public_key_test'] : $this->settings['public_key_live'];
        $private_key = ($this->is_test_env) ? $this->settings['private_key_test'] : $this->settings['private_key_live'];

        $this->api = new Maksekeskus($shop_id, $public_key, $private_key, $this->is_test_env);

    }

    /**
     * Fetches shop configuration from the API and stores the fetched data
     * into the database.
     * 
     * @return void
     */
    public function updatePaymentMethods()
    {
        $shop_config = array();

        $response = array(
            'status'  => 200,
            'message' => 'Payment methods has been successfully updated'
        );

        $request_params = array(
            'environment' => json_encode(array(
                'platform' => 'opencart ' . VERSION,
                'module'   => 'omnivaopen ' . $this->config->get('mk_version')
            ))
        );

        try {
            $shop_config     = $this->api->getShopConfig($request_params);
            $payment_methods = $shop_config->payment_methods;
        } catch (Exception $e) {            
            $response = array(
                'status'  => $e->getCode(),
                'message' => $e->getMessage()
            );
        }

        if (isset($payment_methods->banklinks))          
            $this->model_extension_payment_make_commerce->editPaymentMethodFields($payment_methods->banklinks, 'banklink');
        
        if (isset($payment_methods->cards))
            $this->model_extension_payment_make_commerce->editPaymentMethodFields($payment_methods->cards, 'card');

        $shop_name = (isset($shop_config->name) && strlen($shop_config->name)) ? $shop_config->name : '';
        $current_timestamp = time();

        $this->model_extension_payment_make_commerce->editSettings(array(
                'shop_name' => $shop_name,
                'payment_updated_timestamp' => $current_timestamp
            ), 'shop'
        );

        $response['last_time_updated'] = date('d/m/y | H:i:s', $current_timestamp);

        $this->response->setOutput(json_encode($response));
    }

}
