<?php 

require_once(DIR_SYSTEM . 'library/make_commerce/maksekeskus/vendor/autoload.php');

use Maksekeskus\Maksekeskus;

class ControllerExtensionModuleMakeCommerceSco extends Controller {

    /**
     * Loads settings and sets up properties.
     *
     * @param  array $registry
     * @return void                 
     */
    public function __construct($registry)
    {
        parent::__construct($registry);
        
        $cartTotal = 0;
        $cartContents = array();

        $this->config->load('make_commerce');
        $this->load->model('extension/payment/make_commerce');
        $this->load->model('extension/module/make_commerce_sco');

        $this->payment_methods = $this->model_extension_payment_make_commerce->getPaymentMethods();
        $this->settings        = $this->model_extension_payment_make_commerce->getSettings();
        $this->is_test_env     = $this->settings['api_environment'] !== 'live';

        $shop_id     = ($this->is_test_env) ? $this->settings['shop_id_test'] : $this->settings['shop_id_live'];
        $public_key  = ($this->is_test_env) ? $this->settings['public_key_test'] : $this->settings['public_key_live'];
        $private_key = ($this->is_test_env) ? $this->settings['private_key_test'] : $this->settings['private_key_live'];

        $this->api = new Maksekeskus($shop_id, $public_key, $private_key, $this->is_test_env);
    }
    private function isCurrentUrlHttps() {
        return isset($_SERVER['HTTPS']) && ($_SERVER['HTTPS'] == "on" || $_SERVER["HTTPS"] == 1);
    }

    public function sendCart() {
        $isCurrentUrlHttps = $this->isCurrentUrlHttps();
        $this->load->model('localisation/country');
        $cartId            = $this->model_extension_module_make_commerce_sco->setCartCopy( $this->cart->getProducts() );
        $countryCode = $this->model_localisation_country->getCountry($this->config->get('config_country_id'));
        $cartArray['cartRef']                         = "PreOrder " . $cartId;
        $cartArray['pluginUrls']['cartToOrder']       = ($isCurrentUrlHttps ? HTTPS_SERVER : HTTP_SERVER) . 'index.php?route=extension/payment/make_commerce/callback';
        $cartArray['pluginUrls']['calculateShipment'] = ($isCurrentUrlHttps ? HTTPS_SERVER : HTTP_SERVER) . 'index.php?route=extension/payment/make_commerce/callback';
        $cartArray['pluginUrls']['tos']               = ($isCurrentUrlHttps ? HTTPS_SERVER : HTTP_SERVER) . $this->settings['payment_sco_tos_link'];
        $cartArray['currency']                        = $this->session->data['currency'];
        $cartArray['sourceCountry']                   = $countryCode['iso_code_2'];
        $cartArray['locale']                          = $this->language->get('code');
        if ($cartArray['locale']=="") $cartArray['locale'] = "en";  // english GUI if locale is not set because of some config issue
        $totals = $this->getTotals();
        foreach ($totals  as $key => $value) {
            if ( $value['code'] == 'total') $cartArray['amount'] = number_format( $value['value'], 2);
        }
        unset ($y);
        $x = $this->model_extension_module_make_commerce_sco->getShipmentMethods(); // no method given
        foreach ($x as $value) {
            unset ($value["method"]);
            unset($value["codename"]);
            $y[]=$value;
        }
        $cartArray['shipmentMethods'] = $y;
        ini_set('serialize_precision', 14);
        $resp = $this->api->createCart( $cartArray );
        if (isset($resp->object)) {
            if ($resp->object == 'cart') {
                $this->model_extension_module_make_commerce_sco->updateCartId($cartId, $resp->id);
                $this->response->redirect($resp->scoUrl);
            }
        }
    }
    public function getTotals() {
        $this->load->language('common/cart');

        // Totals
        $this->load->model('setting/extension');
        $totals = array();
        $taxes = $this->cart->getTaxes();
        $total = 0;

        // Because __call can not keep var references so we put them into an array.
        $total_data = array(
            'totals' => &$totals,
            'taxes'  => &$taxes,
            'total'  => &$total
        );

        // Display prices
        if ($this->customer->isLogged() || !$this->config->get('config_customer_price')) {
            $sort_order = array();
            $results = $this->model_setting_extension->getExtensions('total');
            foreach ($results as $key => $value) {
                $sort_order[$key] = $this->config->get('total_' . $value['code'] . '_sort_order');
            }
            array_multisort($sort_order, SORT_ASC, $results);
            foreach ($results as $result) {
                if ($this->config->get('total_' . $result['code'] . '_status')) {
                    $this->load->model('extension/total/' . $result['code']);
                    // We have to put the totals in an array so that they pass by reference.
                    $this->{'model_extension_total_' . $result['code']}->getTotal($total_data);
                }
            }
            $sort_order = array();
            foreach ($totals as $key => $value) {
                $sort_order[$key] = $value['sort_order'];
            }
            array_multisort($sort_order, SORT_ASC, $totals);
            return $totals;
        } else return false;
    }
}
