<?php

/*
 *  location: catalog/extension/shipping
 */

require_once DIR_SYSTEM . 'library/make_commerce/maksekeskus/vendor/autoload.php';

use Maksekeskus\Maksekeskus;

class ControllerExtensionShippingOmnivaParcelByMakeCommerce extends Controller {

    /**
     * Instance of the Maksekeskus SDK class
     */
    private $api;

    /**
     * Module settings
     */
    private $settings;

    /**
     * Is test environment
     */
    private $is_test_env;

    /**
     * Loads settings and sets up properties.
     *
     * @param  array $registry
     * @return void                 
     */
    public function __construct($registry)
    {
        parent::__construct($registry);

        $this->config->load('make_commerce');
        $this->load->model('extension/shipping/omniva_parcel_by_make_commerce');

        $this->settings    = $this->model_extension_shipping_omniva_parcel_by_make_commerce->getSettings();
        $this->is_test_env = $this->settings['api_environment'] !== 'live';

        $shop_id     = ($this->is_test_env) ? $this->settings['shop_id_test'] : $this->settings['shop_id_live'];
        $public_key  = ($this->is_test_env) ? $this->settings['public_key_test'] : $this->settings['public_key_live'];
        $private_key = ($this->is_test_env) ? $this->settings['private_key_test'] : $this->settings['private_key_live'];

        $this->api = new Maksekeskus($shop_id, $public_key, $private_key, $this->is_test_env);         
    }

    /**
     * Sets selected parcel pickup point.
     *
     * @return void                 
     */
    public function set()
    {
        $parcel_machines_regions = $this->model_extension_shipping_omniva_parcel_by_make_commerce->getShippingParcels('omniva');
        $lang                    = (isset($this->session->data['language'])) ? $this->session->data['language'] : $this->config->get('config_language');
        $method_names            = $this->settings['omniva_parcel_method_name_languages'];
        $shipping_title          = (isset($method_names[$lang]) ? $method_names[$lang] : '');
        $shipping_method         = 'omniva_parcel_by_make_commerce';

        $method_data = array(
            'code'       => 'omniva_parcel_by_make_commerce',
            'title'      => $shipping_title,
            'quote'      => $this->getShippingQuoteData($parcel_machines_regions, $shipping_title),
            'sort_order' => $this->settings['omniva_parcel_sort_order'],
            'error'      => ''
        );

        $this->session->data['shipping_methods'][$shipping_method] =  $method_data;

    }

    /**
     * Gets omniva parcel machine shipping quote data.
     *
     * @param  array  $parcel_machines_regions
     * @param  string $shipping_title
     * @return array                 
     */
    private function getShippingQuoteData($parcel_machines_regions, $shipping_title)
    {
        $address      = $this->session->data['shipping_address'];        
        $geo_zone_ids = $this->model_extension_shipping_omniva_parcel_by_make_commerce->getGeoZoneIds($address['country_id'], $address['zone_id']);
        $currency     = $this->session->data['currency'];
        $cost         = $this->currency->convert($this->model_extension_shipping_omniva_parcel_by_make_commerce->getShippingCost($geo_zone_ids), 'EUR', $currency);        
        $quote_data   = array();

        foreach ($parcel_machines_regions as $parcel_machines_region) {
            foreach ($parcel_machines_region as $parcel_machine_city) {
                foreach ($parcel_machine_city as $parcel_machine) {
                    
                    $quote_data[$parcel_machine['parcel_id']] = array(
                        'code'         => 'omniva_parcel:' . $parcel_machine['parcel_id'],
                        'title'        => $shipping_title . ' - ' . $parcel_machine['name'],
                        'cost'         => $cost,
                        'tax_class_id' => $this->settings['omniva_parcel_tax_class'],
                        'text'         => $this->currency->format($this->tax->calculate($cost, $this->settings['omniva_parcel_tax_class'], $this->config->get('config_tax')), $this->session->data['currency'], 1.0000000),
                    );

                }
            }
        }

        return $quote_data;
    }

}
