<?php

require_once DIR_SYSTEM . '../admin/model/extension/module/make_commerce.php';

class ModelExtensionPaymentMakeCommerce extends Model {

    /**
     * Admin model
     */
    private $make_commerce_model;

    /**
     * Module API settings
     */
    private $settings;

    /**
     * Module payment settings
     */
    private $payment_methods_settings;

    /**
     * Model's constructor.
     *
     * @return void
     */
    public function __construct($registry)
    {
        parent::__construct($registry);
        $this->config->load('make_commerce');

        $this->make_commerce_model = new ModelExtensionModuleMakeCommerce($registry);

        if ( ! $this->make_commerce_model->isSettingTableCreated())
            return;

        $this->settings                 = $this->make_commerce_model->getSettings();
        $this->payment_methods_settings = $this->make_commerce_model->getPaymentMethods();
    }

    /**
     * Gets called on the checkout page to display MakeCommerce
     * payment option.
     *
     * @param  array $address
     * @return array
     */
     public function getMethod($address, $total) {
        $this->load->language('payment/make_commerce');

        $payment_method_box_html = $this->getPaymentMethodBoxHtml($address);
        $lang                    = (isset($this->session->data['language'])) ? $this->session->data['language'] : $this->config->get('config_language');
        $method_names            = $this->settings['payments_widget_language_title'];
        $title                   = (isset($method_names[$lang]) ? $method_names[$lang] : '');
        $payment_option_title    = $title . $payment_method_box_html;

        $this->session->data['make_commerce_method'] = ( ! empty($this->session->data['make_commerce_method'])) ? $this->session->data['make_commerce_method'] : '';
        $method_data = array( 
            'code'       => 'make_commerce',
            'title'      => $payment_option_title,
            'terms'      => '',
            'sort_order' => 1
        );
        return $method_data;
    }

    /**
     * Gets an array of MakeCommerce payment methods by calling admin's
     * model (admin/model/extension/module/make_commerce.php) method getPaymentMethods.
     *
     * @return array
     */
    public function getPaymentMethods()
    {
        return $this->make_commerce_model->getPaymentMethods();
    }

    /**
     * Adds a meta value to the given order by calling admin's
     * model (admin/model/extension/module/make_commerce.php) method updateOrderData.
     *
     * @return void
     */
    public function updateOrderData($order_id, $name, $value)
    {
        return $this->make_commerce_model->updateOrderData($order_id, $name, $value);
    }

    /**
     * Deletes a meta value from the given order by calling admin's
     * model (admin/model/extension/module/make_commerce.php) method deleteOrderData.
     *
     * @param  int    $order_id
     * @param  string $name
     * @return void
     */
    public function deleteOrderData($order_id, $name)
    {
        $this->make_commerce_model->deleteOrderData($order_id, $name);
    }

    /**
     * Gets an array of MakeCommerce settings by calling admin's
     * model (admin/model/extension/module/make_commerce.php) method getSettings.
     *
     * @return array
     */
    public function getSettings()
    {
        return $this->make_commerce_model->getSettings();
    }

    /**
     * Edit multiple settings at once by calling admin's
     * model (admin/model/extension/module/make_commerce.php) method editSettings.
     *
     * @param  array $setting_names
     * @param  array $values
     * @return void
     */
    public function editSettings($values, $setting_type = false)
    {
        return $this->make_commerce_model->editSettings($values, $setting_type);
    }

    /**
     * Sets order's payment method to the given one by calling admin's
     * model (admin/model/extension/module/make_commerce.php) method setOrderPaymentMethod.
     *
     * @param  int $order_id
     * @param  int $title
     * @return void
     */
    public function setOrderPaymentMethod($order_id, $title)
    {
        return $this->make_commerce_model->setOrderPaymentMethod($order_id, $title);
    }

    /**
     * Edit multiple payment method settings at once by calling admin's
     * model (admin/model/extension/module/make_commerce.php) method editPaymentMethodFields.
     *
     * @param  array  $fields
     * @param  string $type
     * @return void
     */
    public function editPaymentMethodFields($fields, $type)
    {
        return $this->make_commerce_model->editPaymentMethodFields($fields, $type);
    }

    /**
     * Gets additional html for MakeCommerce payment option.
     *
     * @param  array  $address
     * @param  array  $settings
     * @param  array  $payment_methods_settings
     * @return string
     */
    private function getPaymentMethodBoxHtml($address)
    {
        $settings                 = $this->settings;
        $payment_methods_settings = $this->payment_methods_settings;
        $bank_links               = $payment_methods_settings['banklinks'];
        $cards                    = $payment_methods_settings['cards'];
        $default_country          = $this->getDefaultCountry($address['iso_code_2'], $payment_methods_settings['countries']);

        usort($bank_links, array($this, 'orderBanklinks'));

        $data['payment_image_base_url']   = $this->config->get('payment_image_base_url');
        $data['settings']                 = $settings;
        $data['default_country']          = $default_country;
        $data['payment_methods_settings'] = $payment_methods_settings;
        $data['bank_links']               = $bank_links;
        $data['cards']                    = $cards;
        
        return $this->load->view('make_commerce/payment_option', $data);
    }

    /**
     * Finds default country from the given countries array.
     *
     * @param  string  $country_iso_code
     * @param  array   $available_countries
     * @return string
     */
    private function getDefaultCountry($country_iso_code, $available_countries)
    {
        $country_iso_code = strtolower($country_iso_code);

        if (in_array($country_iso_code, $available_countries))
            return $country_iso_code;
        else
            return 'other';
    }

    /**
     * Sorts banklinks using settings that were defined on MakeCommerce settings page.
     *
     * @param  array  $a
     * @param  array  $b
     * @return int
     */
    private function orderBanklinks($a, $b) {
        $order = array_map('trim', explode(",", $this->settings['custom_order_payment_channels']));

        $pos_a = array_search($a['name'], $order);
        $pos_b = array_search($b['name'], $order);

        if($pos_a === $pos_b) return $a['id'] > $b['id'] ? 1 : -1;
        if($pos_a === FALSE) return 1;
        if($pos_b === FALSE) return -1;

        return $pos_a > $pos_b ? 1 : -1;
    }

}
