<?php
namespace Cronp\Schedule;

use Countable;
use Iterator;

/**
 * Class Collection
 * @package Cronp\Schedule
 * @author Renato Neto
 */
class Collection implements Countable, Iterator
{

    /**
     * @var array
     */
    protected $collection = [];

    /**
     * Return the current element
     * @link http://php.net/manual/en/iterator.current.php
     * @return mixed Can return any type.
     */
    public function current()
    {
        return current($this->collection);
    }

    /**
     * Move forward to next element
     * @link http://php.net/manual/en/iterator.next.php
     * @return void Any returned value is ignored.
     */
    public function next()
    {
        return next($this->collection);
    }

    /**
     * Return the key of the current element
     * @link http://php.net/manual/en/iterator.key.php
     * @return mixed scalar on success, or null on failure.
     */
    public function key()
    {
        return key($this->collection);
    }

    /**
     * Checks if current position is valid
     * @link http://php.net/manual/en/iterator.valid.php
     * @return boolean The return value will be casted to boolean and then evaluated.
     * Returns true on success or false on failure.
     */
    public function valid()
    {
        return key($this->collection) !== null;
    }

    /**
     * Rewind the Iterator to the first element
     * @link http://php.net/manual/en/iterator.rewind.php
     * @return void Any returned value is ignored.
     */
    public function rewind()
    {
        return reset($this->collection);
    }

    /**
     * Count elements of an object
     * @link http://php.net/manual/en/countable.count.php
     * @return int The custom count as an integer.
     * The return value is cast to an integer.
     */
    public function count()
    {
        return count($this->collection);
    }

    /**
     * @param Schedule $schedule
     * @param null $key
     * @return bool
     */
    public function add(Schedule $schedule, $key = null)
    {
        $key = ($key) ?: md5($schedule->getCommand());
        $this->collection[$key] = $schedule;

        return true;
    }

    /**
     * @param Schedule $schedule
     * @param null $key
     * @return bool
     */
    public function remove(Schedule $schedule, $key = null)
    {
        $key = ($key) ?: md5($schedule->getCommand());

        if (!isset($this->collection[$key])) {
            return false;
        }

        unset($this->collection[$key]);
        return true;
    }

}