<?php 

class MakeCommerceOrderShippingDetailExtension {

    /**
     * Module
     */
    private $mk_model;

    /**
     * Module settings
     */
    private $settings;

    /**
     * Loads settings and sets up properties.
     *
     * @param  Object $mk_model
     * @return void                 
     */
    public function __construct($mk_model)
    {
        $this->mk_model = $mk_model;
        $this->settings = $mk_model->getSettings();
    }

    public function setShippingMethodQuoteData(&$shipping_methods)
    {
        foreach ($shipping_methods as $key => $shipping_method) {
            if ($key == 'omniva_parcel_by_make_commerce')
                $shipping_methods[$key]['quote'] = $this->getQuoteData($shipping_method['title'], $shipping_method['quote']['omniva_parcel_by_make_commerce'], 'omniva');
        }
    }

    /**
     * Gets omniva parcel machine shipping quote data.
     *
     * @param  array  $parcel_machines_regions
     * @param  string $shipping_title
     * @return array                 
     */
    private function getQuoteData($shipping_title, $shipping_method, $type)
    {
        $parcel_machines_regions = $this->mk_model->getShippingParcels($type);  
        $code                    = ($type == 'omniva') ? 'omniva_parcel_by_make_commerce.'; 
        $quote_data              = array();           

        foreach ($parcel_machines_regions as $parcel_machines_region) {
            foreach ($parcel_machines_region as $parcel_machine_city) {
                foreach ($parcel_machine_city as $parcel_machine) {
                    
                    $quote_data[$parcel_machine['parcel_id']] = array(
                        'code'         => $code . $parcel_machine['parcel_id'],
                        'title'        => $shipping_title . ' - ' .  $parcel_machine['name'],
                        'cost'         => $shipping_method['cost'],
                        'tax_class_id' => $shipping_method['tax_class_id'],
                        'text'         => $shipping_method['text'],
                    );

                }
            }
        }

        return $quote_data;
    }

}

$this->load->model('extension/shipping/omniva_parcel_by_make_commerce');
$mk_model = $this->model_extension_shipping_omniva_parcel_by_make_commerce;

if ($mk_model->isSettingTableCreated()) {
    $mk_order_detail_extension = new MakeCommerceOrderShippingDetailExtension($mk_model);
    $mk_order_detail_extension->setShippingMethodQuoteData($json['shipping_methods']);
}
